@@ -15,7 +15,7 @@ use warnings;
 use Carp qw(croak);
 use File::KeePass;
 
-our $VERSION = '0.02';
+our $VERSION = '0.03';
 our @ISA;
 BEGIN {
     my $os = lc($^O);
@@ -32,49 +32,116 @@ sub new {
 
 sub run {
     my $self = ref($_[0]) ? shift() : __PACKAGE__->new;
-    my $file = shift || shift(@ARGV) || $self->prompt_for_file || croak "Can't continue without kdb file";
-    my $pass = shift || shift(@ARGV) || $self->prompt_for_pass($file) || croak "Can't continue without master password";
 
-    my $k = $self->keepass;
-    $k->load_db($file, $pass);
-
-    print $k->dump_groups({'group_title !' => 'Backup', 'title !' => 'Meta-Info'});
+    # handle args coming in a multitude of ways
+    my @pairs;
+    if (@_) {
+        my ($files, $passes) = @_;
+        if (ref($_[0]) eq 'ARRAY') {
+            push @pairs, [$files->[$_], $passes->[$_]] for 0 .. $#$files;
+        } elsif (ref($_[0] eq 'HASH')) {
+            push @pairs, map {[$_ => $files->{$_}]} sort keys %$files;
+        } else {
+            push @pairs, [$files, $passes]; # single file/pass set
+        }
+    } elsif (@ARGV) {
+        for (my $i = 0; $i < @ARGV; $i++) {
+            my $file = $ARGV[$i];
+            next if $file =~ /^--?\w+$/;
+            if ($ARGV[$i+1] && $ARGV[$i+1] =~ /^--?pass=(.+)/) {
+                push @pairs, [$file, $1];
+                $i++;
+            } else {
+                push @pairs, [$file, undef];
+            }
+        }
+    } else {
+        my $file = $self->prompt_for_file or die "Cannot continue without kdb file\n";
+        push @pairs, map {[$_, undef]} glob $file;
+    }
+    die "No files given as input\n" if ! @pairs;
 
+    # check file existence
     my @callbacks;
+    for my $pair (@pairs) {
+        my ($file, $pass) = @$pair;
+        die "File \"$file\" does not exist\n" if ! -e $file;
+        die "File \"$file\" does not appear to be readible\n" if ! -r $file;
+        die "File \"$file\" does not appear to be a valid keepass db file\n" if ! -B $file;
+    }
+    OUTER: for my $pair (@pairs) {
+        my ($file, $pass) = @$pair;
+        my $k;
+        if (! defined $pass) {
+            while (!$k) {
+                $pass = $self->prompt_for_pass($file);
+                next if ! defined($pass) || !length($pass);
+                $k = eval { $self->load_keepass($file, $pass) };
+                warn "Could not load database: $@" if ! $k;
+            }
+        } else {
+            $k = $self->load_keepass($file, $pass);
+        }
+    }
+
+    # show what is open
+    my $kdbs = $self->keepass;
+    die "No open databases.\n" if ! @$kdbs;
+    for my $pair (@$kdbs) {
+        my ($file, $kdb) = @$pair;
+        print "$file\n";
+        print $kdb->dump_groups({'group_title !' => 'Backup', 'title !' => 'Meta-Info'})
+    }
+
+    # figure out what to bind
+    foreach my $row ($self->active_entries) {
+        my ($file, $entries) = @$row;
+        foreach my $e (@$entries) {
+            next if ! $e->{'comment'} || $e->{'comment'} !~ /^Custom-Global-Shortcut:\s*(.+?)\s*$/m;
+            my %info = map {lc($_) => 1} split /[\s+-]+/, $1;
+            my %at = $e->{'comment'} =~ m{ ^Auto-Type((?:-\d+)?): \s* (.+?) \s*$ }mxg;
+            next if ! scalar keys %at;
+            my $at = $at{""} || $at{(sort keys %at)[0]};
+            my $s = {
+                ctrl  => delete($info{'control'}) || delete($info{'cntrl'}) || delete($info{'ctrl'}),
+                shift => delete($info{'shift'}) || delete($info{'shft'}),
+                alt   => delete($info{'alt'}),
+                win   => delete($info{'win'}),
+            };
+            my @keys = keys %info;
+            if (@keys != 1) {
+                croak "Cannot set global shortcut with more than one key (@keys) for entry \"$e->{'title'}\"\n";
+            }
+            $s->{'key'} = lc $keys[0];
+            push @callbacks, [$s, sub {
+                my ($self, $title, $event) = @_;
+                return $self->do_auto_type({auto_type => $at, entry => $e, file => $file}, $title, $event);
+            }];
+            print "Listening on ".$self->shortcut_name($s)." for entry $e->{'title'}\n";
+        }
+    }
     if (my $s = $self->read_config('global_shortcut')) {
         push @callbacks, [$s, 'search_auto_type'];
         print "Listening on ".$self->shortcut_name($s)." for global shortcut\n";
     }
-    foreach my $e ($self->active_entries) {
-        next if ! $e->{'comment'} || $e->{'comment'} !~ /^Custom-Global-Shortcut:\s*(.+?)\s*$/m;
-        my %info = map {lc($_) => 1} split /[\s+-]+/, $1;
-        my %at = $e->{'comment'} =~ m{ ^Auto-Type((?:-\d+)?): \s* (.+?) \s*$ }mxg;
-        next if ! scalar keys %at;
-        my $at = $at{""} || $at{(sort keys %at)[0]};
-        my $s = {
-            ctrl  => delete($info{'control'}) || delete($info{'cntrl'}) || delete($info{'ctrl'}),
-            shift => delete($info{'shift'}) || delete($info{'shft'}),
-            alt   => delete($info{'alt'}),
-            win   => delete($info{'win'}),
-        };
-        my @keys = keys %info;
-        if (@keys != 1) {
-            croak "Cannot set global shortcut with more than one key (@keys) for entry \"$e->{'title'}\"\n";
-        }
-        $s->{'key'} = lc $keys[0];
-        push @callbacks, [$s, sub {
-            my ($self, $title, $event) = @_;
-            return $self->do_auto_type($at, $e, $title, $event);
-        }];
-        print "Listening on ".$self->shortcut_name($s)." for entry $e->{'title'}\n";
-    }
     if (! @callbacks) {
         croak "No global_shortcut defined - hiding away for now";
     }
+
     $self->grab_global_keys(@callbacks);
 }
 
-sub keepass { shift->{'keepass'} ||= File::KeePass->new }
+sub load_keepass {
+    my ($self, $file, $pass) = @_;
+    my $kdb = $self->keepass_class->new;
+    $kdb->load_db($file, $pass);
+    push @{ $self->keepass }, [$file, $kdb];
+    return $kdb;
+}
+
+sub keepass { shift->{'keepass'} ||= [] }
+
+sub keepass_class { 'File::KeePass' }
 
 sub shortcut_name {
     my ($self, $s) = @_;
@@ -82,26 +149,38 @@ sub shortcut_name {
     return $mod ? "$mod $s->{'key'}" : $s->{'key'};
 }
 
-sub active_entries { shift->keepass->find_entries({active => 1, 'group_title !' => 'Backup', 'title !' => 'Meta-Info'}) }
+sub active_entries {
+    my $self = shift;
+    my @rows;
+    foreach my $pair (@{ $self->keepass }) {
+        my ($file, $kdb) = @$pair;
+        my @entries = $kdb->find_entries({active => 1, 'group_title !' => 'Backup', 'title !' => 'Meta-Info'});
+        push @rows, [$file, \@entries] if @entries;
+    }
+    return @rows;
+}
 
 sub active_searches {
     my $self = shift;
     my $s = $self->{'active_searches'} ||= do {
         my @s;
-        foreach my $e ($self->active_entries) {
-            next if ! $e->{'comment'};
-            my %at = $e->{'comment'} =~ m{ ^Auto-Type((?:-\d+)?): \s* (.+?) \s*$ }mxg;
-            next if ! scalar keys %at;
-            my @w  = $e->{'comment'} =~ m{ ^Auto-Type-Window((?:-\d+)?): \s* (.+?) \s*$ }mxg;
-            while (@w) {
-                my $n = shift @w;
-                my $t = shift @w;
-                my $at = defined($at{$n}) ? $at{$n}: defined($at{""}) ? $at{""} : next;
-                $t = quotemeta($t);
-                $t =~ s{^\\\*}{.*};
-                $t =~ s{\\\*$}{.*};
-                $t = qr{^$t$};
-                push @s, {'qr' => $t, auto_type => $at, entry => $e};
+        foreach my $row ($self->active_entries) {
+            my ($file, $entries) = @$row;
+            foreach my $e (@$entries) {
+                next if ! $e->{'comment'};
+                my %at = $e->{'comment'} =~ m{ ^Auto-Type((?:-\d+)?): \s* (.+?) \s*$ }mxg;
+                next if ! scalar keys %at;
+                my @w  = $e->{'comment'} =~ m{ ^Auto-Type-Window((?:-\d+)?): \s* (.+?) \s*$ }mxg;
+                while (@w) {
+                    my $n = shift @w;
+                    my $t = shift @w;
+                    my $at = defined($at{$n}) ? $at{$n}: defined($at{""}) ? $at{""} : next;
+                    $t = quotemeta($t);
+                    $t =~ s{^\\\*}{.*};
+                    $t =~ s{\\\*$}{.*};
+                    $t = qr{^$t$};
+                    push @s, {'qr' => $t, auto_type => $at, file => $file, entry => $e};
+                }
             }
         }
         \@s;
@@ -123,7 +202,7 @@ sub search_auto_type {
         $self->do_auto_type_mult(\@matches, $title, $event);
     }
     else {
-        $self->do_auto_type($matches[0]->{'auto_type'}, $matches[0]->{'entry'}, $title, $event);
+        $self->do_auto_type($matches[0], $title, $event);
     }
 }
 
@@ -133,12 +212,13 @@ sub do_no_match {
 }
 
 sub do_auto_type {
-    my ($self, $auto_type, $entry, $title, $event) = @_;
-
+    my ($self, $match, $title, $event) = @_;
+    my ($auto_type, $file, $entry) = @$match{qw(auto_type file entry)};
     $auto_type =~ s{ \{ TAB      \} }{\t}xg;
     $auto_type =~ s{ \{ ENTER    \} }{\n}xg;
     $auto_type =~ s{ \{ PASSWORD \} }{
-        $self->keepass->locked_entry_password($entry);
+        my %kdbs = map {$_->[0], $_->[1]} @{ $self->keepass };
+        $kdbs{$file}->locked_entry_password($entry);
     }xeg;
     $auto_type =~ s{ \{ (\w+)    \} }{
         my $key = lc $1;
@@ -153,7 +233,7 @@ sub do_auto_type {
 sub do_auto_type_mult {
     my ($self, $matches, $title, $event) = @_;
     warn "Found multiple matches - using the first\n";
-    $self->do_auto_type_mult($matches->[0]->{'auto_type'}, $matches->[0]->{'entry'}, $title, $event);
+    $self->do_auto_type($matches->[0], $title, $event);
 }
 
 sub do_auto_type_unsupported {
@@ -170,11 +250,20 @@ __END__
    use File::KeePass::Agent;
    File::KeePass::Agent->new->run($file, $pass);
 
-   # OR
-   File::KeePass::Agent->new->run; # will read from @ARGV or prompt
 
-   # OR
-   File::KeePass::Agent::run(); # will read from @ARGV or prompt
+   File::KeePass::Agent->new->run;  # will read from @ARGV or prompt
+
+
+   File::KeePass::Agent::run();  # will read from @ARGV or prompt
+
+
+   File::KeePass::Agent::run(\%files);  # file/pass pairs
+
+
+   File::KeePass::Agent::run(\@files);
+
+   File::KeePass::Agent::run(\@files, \@passes);  # parallel arrays
+
 
 You may pass the name of the keepass filename that you would like to open.  Otherwise you are prompted
 for the file to open.
@@ -209,7 +298,7 @@ Eventually, this will most likely support more maintenance features.
 
 =item C<keepass>
 
-Returns a File::KeePass object used for reading the database and keeping the entries looked in memory.
+Returns an arrayref of arrayrefs continaing file and File::KeePass object pairs.
 
 =item C<shortcut_name>
 
@@ -217,7 +306,7 @@ Returns a human readable name from a shortcut hashref.
 
 =item C<active_entries>
 
-Finds current active entries from the current database.
+Finds current active entries from any of the open databases.
 
 =item C<active_searches>
 
